CREATE TABLE IF NOT EXISTS events (
    Id INTEGER PRIMARY KEY AUTOINCREMENT,
    expire_date INTEGER,
    kind TEXT,
    payload TEXT,
    start_date INTEGER,
    status TEXT
);

CREATE TABLE IF NOT EXISTS identities (
    Id INTEGER PRIMARY KEY AUTOINCREMENT,
    provider TEXT,
    provider_uid TEXT,
    remote_id TEXT UNIQUE ON CONFLICT REPLACE
);

CREATE TABLE IF NOT EXISTS jaws (
    Id INTEGER PRIMARY KEY AUTOINCREMENT,
    center_x INTEGER,
    center_y INTEGER,
    remote_id TEXT UNIQUE ON CONFLICT REPLACE,
    svg TEXT
);

ALTER TABLE users ADD COLUMN is_paid INTEGER DEFAULT 0;

DROP TABLE characters;
----------------------------------------
-- migrate jaws from heads table
----------------------------------------
ALTER TABLE heads ADD COLUMN jaw INTEGER REFERENCES jaws(Id) ON DELETE NO ACTION ON UPDATE NO ACTION;

CREATE TEMPORARY TABLE jaws_tmp (
    Id INTEGER PRIMARY KEY AUTOINCREMENT,
    center_x INTEGER,
    center_y INTEGER,
    head_id INTEGER
);

INSERT INTO jaws (center_x, center_y)
SELECT
    jaw_center_x, jaw_center_y
FROM heads;

INSERT INTO jaws_tmp (center_x, center_y, head_id) -- to keep head_id reference for later update
SELECT
    jaw_center_x, jaw_center_y, Id
FROM heads;

-- update jaw reference in heads table (sqlite doesn't support UPDATE .. SET = SELECT..., so we use REPLACE INTO)
REPLACE INTO heads (Id, created_at, image_url, jaw, remote_id, updated_at)
SELECT h.Id, h.created_at, h.image_url, h.jaw, h.remote_id, h.updated_at
FROM (
    SELECT heads.Id, heads.created_at, heads.image_url, j.Id AS jaw, heads.remote_id, heads.updated_at
    FROM heads
        LEFT JOIN jaws_tmp j on heads.Id = j.head_id
) h;
DROP TABLE jaws_tmp;

----------------------------------------
-- remove jaw fields from heads table
----------------------------------------
BEGIN TRANSACTION;
CREATE TEMPORARY TABLE heads_backup (
    Id INTEGER PRIMARY KEY AUTOINCREMENT,
    created_at INTEGER,
    image_url TEXT,
    jaw INTEGER REFERENCES jaws(Id) ON DELETE NO ACTION ON UPDATE NO ACTION,
    remote_id TEXT UNIQUE ON CONFLICT REPLACE,
    updated_at INTEGER
);
INSERT INTO heads_backup
SELECT
    Id,
    created_at,
    image_url,
    jaw,
    remote_id,
    updated_at
FROM heads;
DROP TABLE heads;
CREATE TABLE heads(
    Id INTEGER PRIMARY KEY AUTOINCREMENT,
    created_at INTEGER,
    image_url TEXT,
    jaw INTEGER REFERENCES jaws(Id) ON DELETE NO ACTION ON UPDATE NO ACTION,
    remote_id TEXT UNIQUE ON CONFLICT REPLACE,
    updated_at INTEGER
);
INSERT INTO heads(
    Id,
    created_at,
    image_url,
    jaw,
    remote_id,
    updated_at)
SELECT
    Id,
    created_at,
    image_url,
    jaw,
    remote_id,
    updated_at
FROM heads_backup;
DROP TABLE heads_backup;
COMMIT;
