precision   mediump     float;
uniform     sampler2D   u_video_texture;
uniform     sampler2D       u_image_texture;
uniform     float                u_blend_mode;

varying highp vec2 v_video_coordinates;
varying highp vec2 v_image_head_coordinates;

vec3 screen(vec3 source, vec3 destination) {
    return (source + destination) - (source * destination);
}

vec3 multiply(vec3 source, vec3 destination) {
    return source * destination;
}

vec3 add(vec3 source, vec3 destination) {
    return source + destination;
}
vec3 substract(vec3 source, vec3 destination) {
    return destination - source;
}

vec3 darken(vec3 source, vec3 destination) {
    return min(source, destination);
}

vec3 color_burn(vec3 source, vec3 destination) {
    return 1.0 - (1.0 - destination) / source;
}

vec3 linear_burn(vec3 source, vec3 destination) {
    return source + destination - 1.0;
}

vec3 lighten(vec3 source, vec3 destination) {
    return max(source, destination);
}

vec3 color_dodge(vec3 source, vec3 destination) {
    return destination / (1.0 - source);
}

vec3 linear_dodge(vec3 source, vec3 destination) {
    return source + destination;
}

float overlay(float source, float destination) {
    return (destination < 0.5) ? 2.0 * source * destination : 1.0 - 2.0 * (1.0 - source) * (1.0 - destination);
}

vec3 overlay(vec3 source, vec3 destination) {
    vec3 color;
    color.r = overlay(source.r, destination.r);
    color.g = overlay(source.g, destination.g);
    color.b = overlay(source.b, destination.b);
    return color;
}

float soft_light(float source, float destination) {
    return (source < 0.5) ? destination - (1.0 - 2.0 * source) * destination * (1.0 - destination) :
    (destination < 0.25) ? destination + (2.0 * source - 1.0) * destination * ((16.0 * destination - 12.0) * destination + 3.0) :
    destination + (2.0 * source - 1.0) * (sqrt(destination) - destination);
}

vec3 soft_light(vec3 source, vec3 destination) {
    vec3 color;
    color.r = soft_light(source.r, destination.r);
    color.g = soft_light(source.g, destination.g);
    color.b = soft_light(source.b, destination.b);
    return color;
}

float hard_light(float source, float destination) {
    return (source < 0.5) ? 2.0 * source * destination : 1.0 - 2.0 * (1.0 - source) * (1.0 - destination);
}

vec3 hard_light(vec3 source, vec3 destination) {
    vec3 color;
    color.r = hard_light(source.r, destination.r);
    color.g = hard_light(source.g, destination.g);
    color.b = hard_light(source.b, destination.b);
    return color;
}

float vivid_light(float source, float destination) {
    return (source < 0.5) ? 1.0 - (1.0 - destination) / (2.0 * source) : destination / (2.0 * (1.0 - source));
}

vec3 vivid_light(vec3 source, vec3 destination) {
    vec3 color;
    color.r = vivid_light(source.r, destination.r);
    color.g = vivid_light(source.g, destination.g);
    color.b = vivid_light(source.b, destination.b);
    return color;
}

vec3 linear_light(vec3 source, vec3 destination) {
    return 2.0 * source + destination - 1.0;
}

float pin_light(float source, float destination) {
    return (2.0 * source - 1.0 > destination) ? 2.0 * source - 1.0 : (source < 0.5 * destination) ? 2.0 * source : destination;
}

vec3 pin_light(vec3 source, vec3 destination) {
    vec3 color;
    color.r = pin_light(source.r, destination.r);
    color.g = pin_light(source.g, destination.g);
    color.b = pin_light(source.b, destination.b);
    return color;
}

vec3 difference(vec3 source, vec3 destination) {
    return abs(destination - source);
}

vec3 exclusion(vec3 source, vec3 destination) {
    return source + destination - 2.0 * source * destination;
}

float get_min_channel(vec3 color) {
    return min(min(color.r, color.g), color.b);
}

float get_max_channel(vec3 color) {
    return max(max(color.r, color.g), color.b);
}

float lum(vec3 color) {
    return dot(color, vec3(0.30, 0.59, 0.11));
}

float sat(vec3 color) {
    return get_max_channel(color) - get_min_channel(color);
}

vec3 clip_color(vec3 color) {
    float lum = lum(color);
    float min_channel = get_min_channel(color);
    float max_channel = get_max_channel(color);
    if (min_channel < 0.0) {
        color = lum + ((color - lum)*lum) / (lum - min_channel);
    }
    if (max_channel > 1.0) {
        color = lum + ((color - lum)*(1.0 - lum)) / (max_channel - lum);
    }
    return color;
}

vec3 set_lum(vec3 base_color, vec3 lum_color) {
    float lbase = lum(base_color);
    float llum = lum(lum_color);
    float ldiff = llum - lbase;
    vec3 color = base_color + vec3(ldiff);
    return clip_color(color);
}

vec3 set_lum_sat(vec3 base_color, vec3 sat_color, vec3 lum_color) {
    float minbase = get_min_channel(base_color);
    float sbase = sat(base_color);
    float ssat = sat(sat_color);
    vec3 color;
    if (sbase > 0.0) {
        color = (base_color - minbase) * ssat / sbase;
    } else {
        color = vec3(0.0);
    }
    return set_lum(color, lum_color);
}

vec3 saturation(vec3 source, vec3 destination) {
    vec3 result = set_lum_sat(destination, source, destination);
    return result;
}

vec3 hue(vec3 source, vec3 destination) {
    vec3 result = set_lum_sat(source, destination, destination);
    return result;
}

vec3 color(vec3 source, vec3 destination) {
    vec3 result = set_lum(source, destination);
    return result;
}

vec3 luminosity(vec3 source, vec3 destination) {
    vec3 result = set_lum(destination, source);
    return result;
}

vec3 applyBlendMode(vec3 source, vec3 destination, float blendMode) {
    if (blendMode == 0.0) {
        return darken(source.rgb, destination.rgb);
    }

    if (blendMode == 1.0) {
        return multiply(source.rgb, destination.rgb);
    }

    if (blendMode == 2.0) {
        return color_burn(source.rgb, destination.rgb);
    }

    if (blendMode == 3.0) {
        return linear_burn(source.rgb, destination.rgb);
    }

    if (blendMode == 4.0) {
        return add(source.rgb, destination.rgb);
    }

    if (blendMode == 5.0) {
        return lighten(source.rgb, destination.rgb);
    }

    if (blendMode == 6.0) {
        return screen(source.rgb, destination.rgb);
    }

    if (blendMode == 7.0) {
        return color_dodge(source.rgb, destination.rgb);
    }

    if (blendMode == 8.0) {
        return linear_dodge(source.rgb, destination.rgb);
    }

    if (blendMode == 9.0) {
        return overlay(source.rgb, destination.rgb);
    }

    if (blendMode == 10.0) {
        return soft_light(source.rgb, destination.rgb);
    }
    if (blendMode == 11.0) {
        return hard_light(source.rgb, destination.rgb);
    }

    if (blendMode == 12.0) {
        return linear_light(source.rgb, destination.rgb);
    }

    if (blendMode == 13.0) {
        return vivid_light(source.rgb, destination.rgb);
    }

    if (blendMode == 14.0) {
        return pin_light(source.rgb, destination.rgb);
    }

    if (blendMode == 15.0) {
        return difference(source.rgb, destination.rgb);
    }

    if (blendMode == 16.0) {
        return exclusion(source.rgb, destination.rgb);
    }
    if (blendMode == 17.0) {
        return substract(source.rgb, destination.rgb);
    }

    if (blendMode == 18.0) {
        return hue(source.rgb, destination.rgb);
    }

    if (blendMode == 19.0) {
        return saturation(source.rgb, destination.rgb);
    }

    if (blendMode == 20.0) {
        return color(source.rgb, destination.rgb);
    }

    if (blendMode == 21.0) {
        return luminosity(source.rgb, destination.rgb);
    }

    return source;
}

void main() {
    vec4 videoColor = texture2D(u_video_texture, v_video_coordinates);
    vec4 imageColor = texture2D(u_image_texture, v_image_head_coordinates);
    vec3 headEffect = applyBlendMode(imageColor.rgb, videoColor.rgb * imageColor.a, u_blend_mode);
    gl_FragColor = vec4(headEffect.rgb, imageColor.a);
}

