varying highp vec2 v_texPosition0;
uniform sampler2D u_texture;

uniform lowp float u_contrast;
uniform lowp float u_brightness;
uniform lowp float u_lightness;
uniform lowp float u_hue;
uniform lowp float u_saturation;
uniform lowp float u_exposure;
uniform lowp vec4  u_overlayColor;
uniform lowp vec4  u_mixColor;

// Saturation
const mediump vec3 luminanceWeighting = vec3(0.2125, 0.7154, 0.0721);

// Hue
const highp  vec4  kRGBToYPrime = vec4 (0.299, 0.587, 0.114, 0.0);
const highp  vec4  kRGBToI     = vec4 (0.595716, -0.274453, -0.321263, 0.0);
const highp  vec4  kRGBToQ     = vec4 (0.211456, -0.522591, 0.31135, 0.0);

const highp  vec4  kYIQToR   = vec4 (1.0, 0.9563, 0.6210, 0.0);
const highp  vec4  kYIQToG   = vec4 (1.0, -0.2721, -0.6474, 0.0);
const highp  vec4  kYIQToB   = vec4 (1.0, -1.1070, 1.7046, 0.0);

void main()
{
    lowp vec4 color = texture2D(u_texture, v_texPosition0);

    ///////////////////////////////////////////////////
    // Hue
    highp float   YPrime = dot(color, kRGBToYPrime);
    highp float   I      = dot(color, kRGBToI);
    highp float   Q      = dot(color, kRGBToQ);

    // Calculate the hue and chroma
    highp float   hue     = atan(Q, I);
    highp float   chroma  = sqrt(I * I + Q * Q);

    // adjust hue
    hue += u_hue;

    // Convert back to YIQ
    Q = chroma * sin(hue);
    I = chroma * cos(hue);

    // Convert back to RGB
    highp vec4    yIQ   = vec4(YPrime, I, Q, 0.0);
    color.r = dot(yIQ, kYIQToR);
    color.g = dot(yIQ, kYIQToG);
    color.b = dot(yIQ, kYIQToB);

    ///////////////////////////////////////////////////
    // Saturation
    lowp float luminance = dot(color.rgb, luminanceWeighting);
    lowp vec3 greyScaleColor = vec3(luminance);
    color.rgb = mix(greyScaleColor, color.rgb, u_saturation);

    ///////////////////////////////////////////////////
    // Contrast, Brightness and Lightness
    color.rgb = (color.rgb - 0.5) * u_contrast + 0.5;
    color.rgb += u_brightness;
    color.rgb *= u_lightness;

    ///////////////////////////////////////////////////
    // Exposure
    color.rgb *= pow(2.0, u_exposure);

    ///////////////////////////////////////////////////
    // Mix Color
    color.rgb = mix(color.rgb, u_mixColor.rgb, u_mixColor.a);

    ///////////////////////////////////////////////////
    // Overlay
    lowp vec3 colorStep = step(0.5, color.rgb);
    lowp vec3 overlay = (1.0 - colorStep) * (2.0 * color.rgb * u_overlayColor.rgb);
    overlay += colorStep * (1.0 - 2.0 * (1.0 - color.rgb) * (1.0 - u_overlayColor.rgb));
    color.rgb = mix(color.rgb, overlay, u_overlayColor.a);

    ///////////////////////////////////////////////////
    // Render
    gl_FragColor = color;
}
